@@ -4,12 +4,13 @@ use v5.14;
 use warnings;
 use utf8;
 
-our $VERSION = "2.1301";
+our $VERSION = "2.14";
 
 use Data::Dumper;
 $Data::Dumper::Sortkeys = 1;
 use Carp;
 use List::Util qw(pairmap pairgrep);
+use Scalar::Util qw(looks_like_number);
 use Text::VisualWidth::PP 'vwidth';
 
 ######################################################################
@@ -135,32 +136,43 @@ our %TABSTYLE = (
 
     );
 
+my @default = (
+    text      => '',
+    width     => undef,
+    padding   => 0,
+    boundary  => '',
+    padchar   => ' ',
+    prefix    => '',
+    ambiguous => 'narrow',
+    margin    => 0,
+    linebreak => $DEFAULT_LINEBREAK,
+    runin     => $DEFAULT_RUNIN_WIDTH,
+    runout    => $DEFAULT_RUNOUT_WIDTH,
+    expand    => 0,
+    tabstop   => 8,
+    tabhead   => ' ',
+    tabspace  => ' ',
+    discard   => {},
+    );
+
 sub new {
     my $class = shift;
-    my $obj = bless {
-	text      => '',
-	width     => undef,
-	padding   => 0,
-	boundary  => '',
-	padchar   => ' ',
-	prefix    => '',
-	ambiguous => 'narrow',
-	margin    => 0,
-	linebreak => $DEFAULT_LINEBREAK,
-	runin     => $DEFAULT_RUNIN_WIDTH,
-	runout    => $DEFAULT_RUNOUT_WIDTH,
-	expand    => 0,
-	tabstop   => 8,
-	tabhead   => ' ',
-	tabspace  => ' ',
-	discard   => {},
-    }, $class;
-
+    my $obj = bless { @default }, $class;
     $obj->configure(@_) if @_;
-
     $obj;
 }
 
+INTERNAL_METHODS: {
+    sub spawn {
+	my $obj = shift;
+	my $class = ref $obj;
+	my %new = ( %$obj, pairgrep { defined $b } @_ );
+	bless \%new, $class;
+    }
+    sub do_runin  { $_[0]->{linebreak} & LINEBREAK_RUNIN  && $_[0]->{runin}  > 0 }
+    sub do_runout { $_[0]->{linebreak} & LINEBREAK_RUNOUT && $_[0]->{runout} > 0 }
+}
+
 use Text::ANSI::Fold::Japanese::W3C qw(%prohibition);
 
 sub chars_to_regex {
@@ -190,10 +202,10 @@ my %prohibition_re = do {
 };
 
 sub configure {
-    my $obj = ref $_[0] ? $_[0] : do {
-	state $private = __PACKAGE__->new;
-    };
-    shift;
+    my $obj = shift;
+    if (not ref $obj) {
+	$obj = state $private = __PACKAGE__->new;
+    }
     croak "invalid parameter" if @_ % 2;
     while (@_ >= 2) {
 	my($a, $b) = splice @_, 0, 2;
@@ -229,40 +241,33 @@ sub pop_reset {
     @reset ? do { @color_stack = (); pop @reset } : '';
 }
 
+use constant MAX_INT => ~0 >> 1;
+
 sub fold {
-    my $obj = ref $_[0] ? $_[0] : do {
-	state $private = configure();
-    };
-    shift;
-
+    my $obj = shift;
     local $_ = shift // '';
-    my %opt = ( %$obj, pairgrep { defined $b } @_ );
 
-    my $width = $opt{width} // die;
-    if ($width < 0) {
-	$width = ~0 >> 1; # INT_MAX
+    if (not ref $obj) {
+	$obj = state $private = configure();
     }
+    my $opt = $obj->spawn(splice @_);
 
-    if (not defined $width or $width < 1) {
-	croak "invalid width";
-    }
-
-    if ($width <= $opt{margin}) {
-	croak "invalid margin";
-    }
-    $width -= $opt{margin};
+    my $width = $opt->{width};
+    croak "invalid width" if not looks_like_number $width;
+    $width = MAX_INT if $width < 0;
+    ($width -= $opt->{margin}) > 0 or croak "margin too big";
 
     my $word_char_re =
 	    { word => $alphanum_re, space => $nonspace_re }
-	    ->{$opt{boundary} // ''};
+	    ->{$opt->{boundary} // ''};
 
-    $Text::VisualWidth::PP::EastAsian = $opt{ambiguous} eq 'wide';
+    $Text::VisualWidth::PP::EastAsian = $opt->{ambiguous} eq 'wide';
 
     my $folded = '';
     my $eol = '';
     my $room = $width;
     @bg_stack = @color_stack = @reset = ();
-    my $yield_re = $opt{expand} ? qr/[^\e\n\f\r\t]/ : qr/[^\e\n\f\r]/;
+    my $yield_re = $opt->{expand} ? qr/[^\e\n\f\r\t]/ : qr/[^\e\n\f\r]/;
 
   FOLD:
     while (length) {
@@ -312,9 +317,9 @@ sub fold {
 	}
 
 	# tab
-	if ($opt{expand} and s/\A\t//) {
-	    my $space = $opt{tabstop} - ($width - $room) % $opt{tabstop};
-	    $_ = $opt{tabhead} . $opt{tabspace} x ($space - 1) . $_;
+	if ($opt->{expand} and s/\A\t//) {
+	    my $space = $opt->{tabstop} - ($width - $room) % $opt->{tabstop};
+	    $_ = $opt->{tabhead} . $opt->{tabspace} x ($space - 1) . $_;
 	    next;
 	}
 
@@ -380,28 +385,29 @@ sub fold {
     ##
     ## RUN-OUT
     ##
-    if ($_ ne ''
-	and $opt{linebreak} & LINEBREAK_RUNOUT and $opt{runout} > 0
-	and $folded =~ m{ (?<color>  (?! ${reset_re}) ${color_re}*+ )
+    if ($_ ne '' and $opt->do_runout) {
+	if ($folded =~ m{ (?<color>  (?! ${reset_re}) ${color_re}*+ )
 			  (?<runout> $prohibition_re{end}+ ) \z }xp
-	and ${^PREMATCH} ne ''
-	and (my $w = vwidth $+{runout}) <= $opt{runout}) {
-	$folded = ${^PREMATCH};
-	$_ = join '', ${^MATCH}, @reset, $_;
-	pop_reset() if $+{color};
-	$room += $w;
+	    and ${^PREMATCH} ne ''
+	    and (my $w = vwidth $+{runout}) <= $opt->{runout}) {
+
+	    $folded = ${^PREMATCH};
+	    $_ = join '', ${^MATCH}, @reset, $_;
+	    pop_reset() if $+{color};
+	    $room += $w;
+	}
     }
 
     $folded .= pop_reset() if @reset;
 
-    $room += $opt{margin};
+    $room += $opt->{margin};
 
     ##
     ## RUN-IN
     ##
-    if ($opt{linebreak} & LINEBREAK_RUNIN and $opt{runin} > 0) {
+    if ($opt->do_runin) {
 	my @runin;
-	my $m = $opt{runin};
+	my $m = $opt->{runin};
 	while ($m > 0 and
 	       m{\A (?<color> ${color_re}*+)
 	            (?<runin> $prohibition_re{head})
@@ -423,15 +429,15 @@ sub fold {
 	$_ = join '', @color_stack, $_ if $_ ne '';
     }
 
-    if ($opt{padding} and $room > 0) {
-	my $padding = $opt{padchar} x $room;
+    if ($opt->{padding} and $room > 0) {
+	my $padding = $opt->{padchar} x $room;
 	if (@bg_stack) {
 	    $padding = join '', @bg_stack, $padding, SGR_RESET;
 	}
 	$folded .= $padding;
     }
 
-    if (length and my $p = $opt{prefix}) {
+    if (length and my $p = $opt->{prefix}) {
 	my $s = ref $p eq 'CODE' ? &$p : $p;
 	$_ = $s . $_;
     }
@@ -465,12 +471,17 @@ sub simple_fold {
 }
 
 ######################################################################
+# EXTERNAL METHODS
 
-sub text {
+sub text :lvalue {
     my $obj = shift;
-    croak "Invalid argument" unless @_;
-    $obj->{text} = shift;
-    $obj;
+    if (@_ == 0) {
+	$obj->{text};
+    } else {
+	croak "Invalid argument" if @_ > 1;
+    	$obj->{text} = shift;
+	$obj;
+    }
 }
 
 sub retrieve {
@@ -523,7 +534,7 @@ Text::ANSI::Fold - Text folding library supporting ANSI terminal sequence and As
 
 =head1 VERSION
 
-Version 2.1301
+Version 2.14
 
 =head1 SYNOPSIS
 
@@ -563,6 +574,10 @@ combining characters properly.  Japanese text formatting with
 head-or-end of line prohibition character is also supported.  Set
 the linebreak mode to enable it.
 
+Since the overhead of ANSI escape sequence handling is not significant
+when the data does not include them, this module can be used for
+normal text processing with little penalty.
+
 Use exported B<ansi_fold> function to fold original text, with number
 of visual columns you want to cut off the text.
 
@@ -625,7 +640,7 @@ saved value.
 
 =head1 STRING OBJECT INTERFACE
 
-Fold object can hold string inside by B<text> method.
+A fold object can hold string inside by B<text> method.
 
     $f->text("text");
 
@@ -637,13 +652,14 @@ empty string if nothing remained.
         print "\n" if $folded !~ /\n\z/;
     }
 
-Method B<chops> returns chopped string list.  Because B<text> method
-returns the object itself, you can use B<text> and B<chops> like this:
+Method B<chops> returns chopped string list.  Because the B<text>
+method returns the object itself when called with a parameter, you can
+use B<text> and B<chops> in series:
 
-    print join "\n", $f->text($text)->chops;
+    print join "\n", $f->text($string)->chops;
 
 Actually, text can be set by B<new> or B<configure> method through
-B<text> option.  Next program just works.
+B<text> parameter.  Next program just works.
 
     use Text::ANSI::Fold;
     while (<>) {
@@ -658,10 +674,11 @@ reference, and chops text into given width list.
     my @list = $fold->text("1223334444")->chops(width => [ 1, 2, 3 ]);
     # return ("1", "22", "333") and keep "4444"
 
-If the width value is 0, it returns empty string.
+If the width value is 0, it returns empty string.  Negative width
+value takes all the rest of stored string.
 
-Negative width value takes all the rest of holded string in
-B<retrieve> and B<chops> method.
+Method B<text> has an lvalue attribute, so it can be assigned to, as
+well as can be a subject of mutating operator such as C<s///>.
 
 =head1 OPTIONS
 
@@ -755,7 +772,7 @@ If the trimmed text end with prohibited characters (e.g. opening
 parenthesis), they are ran-out to the head of next line, if it fits to
 maximum width.
 
-Default B<linebreak> mode is B<LINEBREAK_NONE> and can be set one of
+Default B<linebreak> mode is C<LINEBREAK_NONE> and can be set one of
 those:
 
     LINEBREAK_NONE
@@ -763,7 +780,7 @@ those:
     LINEBREAK_RUNOUT
     LINEBREAK_ALL
 
-Import-tag B<:constants> can be used to access these constants.
+Import-tag C<:constants> can be used to access these constants.
 
 Option B<runin> and B<runout> is used to set maximum width of moving
 characters.  Default values are both 2.
@@ -827,8 +844,8 @@ tabspace.
 
 =head1 EXAMPLE
 
-Next code implements almost perfect fold command for multi byte
-characters with prohibited character handling.
+Next code implements almost fully-equipped fold command for multi byte
+text with Japanese prohibited character handling.
 
     #!/usr/bin/env perl
     
@@ -910,7 +927,7 @@ Kazumasa Utashiro
 
 =head1 LICENSE
 
-Copyright 2018- Kazumasa Utashiro.
+Copyright 2018-2022 Kazumasa Utashiro.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
